from loguru import logger

trace = logger.trace
debug = logger.debug
info = logger.info
success = logger.success
warning = logger.warning
error = logger.error
critical = logger.critical
exception = logger.exception
log = logger.log

